<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\MAN\Services;
use Teknisa\MAN\Utilities;

class EnvioCupomEmailController extends \Teknisa\MAN\Controllers\TeknisaCrud {
    protected $envioCupomEmailService;
    protected $utilService;
    protected $dataSourceName = "smtp";
    protected $parameterBagColumns = array('NRORG');
	protected $SMTPService;
    
    public function __construct(Services\EnvioCupomEmailService $EnvioCupomEmailService, Utilities\Util $utilService, Services\SMTPManagement $SMTPService) {
        $this->envioCupomEmailService = $EnvioCupomEmailService;
        $this->utilService = $utilService;
        $this->SMTPService = $SMTPService;
    }
    
    public function findSales (DTO\Request\Filter $request, DTO\Response $response) {
        $conditions = $this->utilService->getParamsAsArray($request);
        
        if (isset($conditions["STATUSENVIO"])) {
            $conditions["STATUSENVIO"] = implode(",", $conditions["STATUSENVIO"]);
        }
        
        if (isset($conditions["DTVENDA"]["start"]) && isset($conditions["DTVENDA"]["end"])) {
            $conditions["DTVENDA"][0] = $conditions["DTVENDA"]["start"];
            $conditions["DTVENDA"][1] = $conditions["DTVENDA"]["end"];
        }
        
        if (!isset($conditions["DTVENDA"])) {
            $conditions["DTVENDA"][0] = date("d/m/Y");
            $conditions["DTVENDA"][1] = date("d/m/Y", strtotime("-1 year"));
        }

        foreach ($conditions as $key => &$value) {
            if (is_array($value) && $key != "DTVENDA") {
                $value = implode(",", $value);
            }
        }
        
        // build params
        $params = array(
            "P_CDFILIAL" => $conditions["CDFILIAL"] ?? "T",
	        "P_CDCONS" => $conditions["CDCONSUMIDOR"] ?? "T",
	        "P_CDCAIXA" => $conditions["CDCAIXA"] ?? "T",
	        "P_CDLOJA" => $conditions["CDLOJA"] ?? "T",
	        "P_DTINICIAL" => $conditions["DTVENDA"][0],
	        "P_DTFINAL" => $conditions["DTVENDA"][1],
	        "P_DSEMAILCUPOM" => $conditions["DSEMAILCUPOM"] ?? "T",
	        "P_DSCHAVE" => $conditions["DSCHAVE"] ?? "T",
	        "P_DSVENDA" => $conditions["NRSEQVENDA"] ?? "T",
	        "P_STATUSENVIO" => $conditions["STATUSENVIO"] ?? "E,U,N,A",
        );
        
        $result = $this->envioCupomEmailService->findSales($params);
        
        $response->addDataSet(new DataSet('vnd08600_envioCupomEmail', $result));
    }
    
    public function sendEmail (DTO\Request $request, DTO\Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            
            $NRORG = $rows[0]["NRORG"];
            $smtp_config = $this->envioCupomEmailService->getSMTPConfig($NRORG);
            
            $messageToUser = $smtp_config[0]['DSMSGCUPOMEMAIL'] ?? '';
            $sender = '';
            
            $sent = true;
            foreach ($rows as $row) {
                $recipientEmail = $row['DSEMAILENVCUPOM'];
                $recipientName = $row['NMCONSUMIDOR'] ?? '';
                $accessKey = $row['NRACESSONFCE'];
        
                $subject = 'Chave de acesso de Cupom Fiscal - Teknisa';
                
                $content = 
                    '<p>Prezado(a) ' . $recipientName . ', segue chave de acesso referente ao cupom fiscal no restaurante:<br>' .
                    $accessKey . ' <br>' .
                    $messageToUser;
                    
                // atualiza status no banco para "aguardando envio"
                $this->envioCupomEmailService->updateStatusEmail('A', 'Aguardando envio.', $row['CDFILIAL'], $row['CDCAIXA'], $row['NUMSEQVENDA']);
                
                $result = $this->SMTPService->sendRawEmail($recipientEmail, $recipientEmail, $sender, $subject, $content);

                if (!$result) {
                    $sent = false;
                    // atualiza banco para não enviado com observação
                    $this->envioCupomEmailService->updateStatusEmail('N', 'Email não enviado.', $row['CDFILIAL'], $row['CDCAIXA'], $row['NUMSEQVENDA']);
                } else {
                    // atualiza banco para sucesso
                    $this->envioCupomEmailService->updateStatusEmail('E', 'Email enviado com sucesso.', $row['CDFILIAL'], $row['CDCAIXA'], $row['NUMSEQVENDA']);
                }
            }
            
            // verifica se algum email nao foi enviado
            if (!$sent) {
                $response->setStatus(Response::STATUS_WARNING);
                $response->addMessage(new Message('Email não enviado. Favor conferir a sua configuração de SMTP.'));
            } else {
                $response->addMessage(new Message('Email(s) enviado(s) com sucesso.'));
                $response->setStatus(Response::STATUS_SUCCESS);
            }
        } catch (\Exception $e) {
            $response->setStatus(Response::STATUS_WARNING);
            $response->addMessage(new Message('Ocorreu um erro inesperado. Favor conferir a sua configuração de SMTP.'));
            $response->addMessage(new Message($e->getMessage()));
        }
    }
    
    public function updateEmail (DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $this->envioCupomEmailService->updateRecipient($row['DSEMAILENVCUPOM'], $row['CDFILIAL'], $row['CDCAIXA'], $row['NUMSEQVENDA']);
        $response->addMessage(new Message('Destinatário atualizado.'));
    }
    
}